#include <cctype>
#include <cstdio>
#include <cstring>
#include <map>

#include "config.hh"
#include "logfun.hh"

int Verbose = 0;

class Cfg : public map<string, string>
{
public:
    Cfg(const string &fn);
};

Cfg::Cfg(const string &fn)
{
    FILE *fp = fopen(fn.c_str(), "rt");
    if(!fp)return;
    char Buf[8192];
    if(Verbose)
    {
        DumpTime();
        fprintf(stderr, "Reading configuration file %s...\n", fn.c_str());
    }
    for(;;)
    {
        if(!fgets(Buf, sizeof Buf, fp))break;
        strtok(Buf, "\n");
        if(!isalpha(Buf[0]))continue;
        string s = Buf;
        unsigned a = s.find(':');
        if(a == s.npos)continue;
        string var = s.substr(0, a);
        while(++a < s.size() && s[a]==' ');
        string content = s.substr(a);
        operator[] (var) = content;
        if(Verbose)
        {
            DumpTime();
            fprintf(stderr, "- %s = '%s'\n",
                var.c_str(),
                content.c_str());
        }
    }
    fclose(fp);
}

Config::Config() : p(NULL)
{
}
Config::~Config()
{
    if(p)delete p;
}
void Config::Rehash()
{
	if(p) { delete p; p=NULL; }
}
const string Config::GetString(const string &name)
{
    if(!p)p = new Cfg(CONFIG);
    return (*p)[name];
}
const unsigned Config::GetInteger(const string &name)
{
    if(!p)p = new Cfg(CONFIG);
    const string &s = (*p)[name];
    unsigned a, retval=0;
    for(a=0; a<s.size(); ++a)
        if(isdigit(s[a]))
            retval = retval*10 + (s[a]-'0');
    return retval;
}
const bool Config::GetBool(const string &name)
{
    if(!p)p = new Cfg(CONFIG);
    string s = (*p)[name];
    unsigned a;
    for(a=0; a<s.size(); ++a)s[a]=toupper(s[a]);
    if(s == "YEP" || s == "YES" || s == "TRUE")return true;
    return false;
}

Config Configuration;
